/* THPSMOTD by Andrew "CHCNiZ" Learn 
 * Copyright (C) 2010 Andrew Learn
 *
 *  THPSMOTD is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  THPSMOTD is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with THPSMOTD.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "chc_endian.h"
char msg[256];
struct in_addr addr;
typedef	struct _THPSMOTD {
	uint32_t checksum;
	uint32_t size;
	uint8_t *msg;
	uint8_t *fbuff;//final buffer
	uint8_t errcode;
	uint8_t *host;//the host for the IP(thps3/4 support.. works on THUG1 too but isn't used)
	uint32_t ip;//resolved IP
	HWND window;//for nice error showing when needed..
} THPSMOTD;
enum {
	THPSERR_SUCCESS,
	THPSERR_PARSEFAIL,
	THPSERR_ALLOCFAIL,
	THPSERR_BADVAR, //unallocated pointers being sent(except for motd because it can't send the variable)
	THPSERR_UNKNOWN,
};
uint32_t resolv(char *host);
void gen_motd(THPSMOTD *motd,int type) {
	unsigned int padlen,i;
	unsigned char msg[1024];
	if(motd==NULL) {
		return;//can't send a error code for this
	}
	if(motd->msg==NULL) {
		motd->errcode=THPSERR_BADVAR;
		return;
	}
	if(type==0) { //THUG2
		motd->size=strlen((const char *)motd->msg);
		motd->fbuff=(uint8_t *)malloc((sizeof(uint32_t)*2)+motd->size+12);
		if(motd->fbuff==NULL) {
			motd->errcode=THPSERR_ALLOCFAIL;
			return;
		}
		motd->msg[++motd->size] ='\x00';
		padlen = (4 - (motd->size % 4));
			for(i = 0; i < padlen; i++) {
				motd->msg[motd->size +i] = '\xCD';
		
			}
		motd->size += (padlen + 2);
		motd->msg[motd->size-2] ='\x01';
		motd->msg[motd->size-1]='\x00';
		motd->checksum=THPSCRC((const char *)motd->msg,motd->size);
		memcpy((void *)motd->fbuff,(void *)&motd->checksum,sizeof(uint32_t));
		memcpy((void *)(motd->fbuff+4),(void *)&motd->size,sizeof(uint32_t));
		memcpy((void *)(motd->fbuff+8),(void *)motd->msg,motd->size);
		motd->size+=(sizeof(uint32_t)*2)+4;
		for(i = 0; i < motd->size; i++)	{
		motd->fbuff[i] ^= (char)0x6D; //THUG2's "encryption"
		}
		motd->errcode=THPSERR_SUCCESS;
	} else {
		motd->checksum=THPSCRC((const char *)&motd->msg,strlen((const char *)motd->msg));
		/*	Your probably curious what the next few lines are even for, but THPS3, and maybe 4 use the "2nd checksum"
			as I called it in previous releases, as the IP it should use for the master server
			Basically, its just a 32bit integer that is the IP it should use for the master server
		*/
		if(motd->host!=NULL) {
			motd->ip=resolv(motd->host);
		} else {
			motd->ip=resolv("master.gamespy.com");
		}
		if(motd->ip==0) { //failed to resolve/not connected to internet
			MessageBox(motd->window?motd->window:NULL,"Warning: Could not retrieve IP of master server!","Warning",MB_OK|MB_ICONWARNING);
			motd->ip=(uint32_t)-1375000881; //fail safe.. the ip of it(obviously resolved manually)
		}
		addr.S_un.S_addr=motd->ip;
		sprintf((char *)&msg,"%d %s",motd->ip,(const char *)motd->msg);
		motd->checksum=THPSCRC((const char *)&msg,strlen((char *)&msg));
		motd->size=strlen((const char *)&msg)+32;
		motd->fbuff=(uint8_t *)malloc(motd->size);
		if(motd->fbuff==NULL) {
			motd->errcode=THPSERR_ALLOCFAIL;
			return;
		}
		sprintf((char *)motd->fbuff,"%d %s",motd->checksum,(const char *)&msg);
		motd->size=strlen((const char *)motd->fbuff);
		motd->errcode=THPSERR_SUCCESS;

	}
}
void read_motd(THPSMOTD *motd,int game) {
	unsigned int i;
	unsigned char *p;
	if(game==0) { //thug2
	for(i=0;i<motd->size;i++) {
		motd->fbuff[i]^=(char)0x6D;
	}
	motd->checksum=*(uint32_t *)motd->fbuff;
	motd->size=*(uint32_t *)(motd->fbuff+sizeof(uint32_t));
	motd->msg=(motd->fbuff+(sizeof(uint32_t)*2));
	motd->errcode=THPSERR_SUCCESS;
	} else { //PRE-THUG2
		*(motd->fbuff+motd->size)=0;
		
		p=motd->fbuff;
		p=(unsigned char *)strchr((char *)motd->fbuff,' ');
		if(!p) {
			motd->errcode=THPSERR_PARSEFAIL;
			return;
		}
		*p=0;
		motd->checksum=atoi((char *)motd->fbuff);
		p=(p+1);
		motd->size=atoi((const char *)(p)); //MOTD checksum
			p=(unsigned char *)strchr((char *)p,' ');
			if(!p) {
				motd->errcode=THPSERR_PARSEFAIL;
				return;
			}
		motd->msg=(uint8_t*)(p+1);
		motd->errcode=THPSERR_SUCCESS;
	}

}
void show_errcode(uint8_t errtype, HWND wnd) {
					char msg[1024];
		switch(errtype) {
			case THPSERR_PARSEFAIL:
				MessageBox(wnd,"Error parsing file!","Error",MB_OK|MB_ICONERROR);
				break;
			case THPSERR_ALLOCFAIL:
				MessageBox(wnd,"Error allocating memory in a MOTD function","Error",MB_OK|MB_ICONERROR);
				break;
			case THPSERR_BADVAR:
				MessageBox(wnd,"Unallocated data in motd structure","Error",MB_OK|MB_ICONERROR);
				break;
			case THPSERR_UNKNOWN:
				MessageBox(wnd,"Unknown Error!","Error",MB_OK|MB_ICONERROR);
				break;
			case THPSERR_SUCCESS:
				break;
			default:
				sprintf(msg,"Unknown Error Code %i!\n",errtype);
				MessageBox(wnd,msg,"Error",MB_OK|MB_ICONERROR);
				break;
		}
}
uint32_t resolv(char *host) {
    struct  hostent *hp;
    uint32_t     host_ip;
    host_ip = inet_addr(host);
    if(host_ip == INADDR_NONE) {
        hp = gethostbyname(host);
        if(!hp) {
			return 0;
        }
        host_ip = *(uint32_t *)hp->h_addr;
    }
    return(host_ip);
}

